//
//  EventModelSpec.swift
//  Octowire
//
//  Created by Mart Roosmaa on 26/02/2017.
//  Copyright © 2017 Mart Roosmaa. All rights reserved.
//

import Foundation
import Quick
import Nimble
import ObjectMapper
@testable import Octowire

class EventModelSpec: QuickSpec {
    override func spec() {
        describe("json parsing") {
            it("can parse create events") {
                let event = EventModel(JSONString: rawCreateRepoJson)
                expect(event).to(beAnInstanceOf(CreateEventModel.self))
                if let event = event as? CreateEventModel {
                    expect(event.id).to(equal("5391454461"))
                    expect(event.type).to(equal("CreateEvent"))
                    expect(event.createdAt).toNot(beNil())
                    expect(event.repoId).to(equal(83169775))
                    expect(event.repoName).to(equal("happylance/ForecastIOClient"))
                    expect(event.actorId).to(equal(4953175))
                    expect(event.actorUsername).to(equal("happylance"))
                    expect(event.actorAvatarUrl).to(equal(URL(string: "https://avatars.githubusercontent.com/u/4953175?")!))
                    
                    expect(event.refType).to(equal(CreateEventModel.RefType.repository))
                }
            }
            
            it("can parse pull request events") {
                let event = EventModel(JSONString: rawPullRequestOpenedJson)
                expect(event).to(beAnInstanceOf(PullRequestEventModel.self))
                if let event = event as? PullRequestEventModel {
                    expect(event.id).to(equal("5391495779"))
                    expect(event.type).to(equal("PullRequestEvent"))
                    expect(event.createdAt).toNot(beNil())
                    expect(event.repoId).to(equal(2928944))
                    expect(event.repoName).to(equal("Azure/azure-sdk-for-net"))
                    expect(event.actorId).to(equal(20732208))
                    expect(event.actorUsername).to(equal("shahabhijeet"))
                    expect(event.actorAvatarUrl).to(equal(URL(string: "https://avatars.githubusercontent.com/u/20732208?")!))
                    
                    expect(event.action).to(equal(PullRequestEventModel.ActionKind.opened))
                    expect(event.pullRequestNumber).to(equal(2861))
                }
            }
            
            it("can parse watch events") {
                let event = EventModel(JSONString: rawWatchJson)
                expect(event).to(beAnInstanceOf(WatchEventModel.self))
                if let event = event as? WatchEventModel {
                    expect(event.id).to(equal("5391454441"))
                    expect(event.type).to(equal("WatchEvent"))
                    expect(event.createdAt).toNot(beNil())
                    expect(event.repoId).to(equal(30161665))
                    expect(event.repoName).to(equal("infernojs/inferno"))
                    expect(event.actorId).to(equal(2153342))
                    expect(event.actorUsername).to(equal("chantron"))
                    expect(event.actorAvatarUrl).to(equal(URL(string: "https://avatars.githubusercontent.com/u/2153342?")!))
                }
            }
            
            it("can parse fork events") {
                let event = EventModel(JSONString: rawForkJson)
                expect(event).to(beAnInstanceOf(ForkEventModel.self))
                if let event = event as? ForkEventModel {
                    expect(event.id).to(equal("5391468192"))
                    expect(event.type).to(equal("ForkEvent"))
                    expect(event.createdAt).toNot(beNil())
                    expect(event.repoId).to(equal(18710823))
                    expect(event.repoName).to(equal("mekkablue/MakeCorner"))
                    expect(event.actorId).to(equal(1623538))
                    expect(event.actorUsername).to(equal("CNXTEoE"))
                    expect(event.actorAvatarUrl).to(equal(URL(string: "https://avatars.githubusercontent.com/u/1623538?")!))

                    expect(event.forkRepoName).to(equal("MakeCorner"))
                }
            }
            
            it("can parse an array of events") {
                let rawJson = "[\(rawCreateRepoJson), \(rawCreateTagJson), {}, \(rawPullRequestOpenedJson), \(rawWatchJson), \(rawForkJson)]"
                let events = [EventModel](JSONString: rawJson)
                expect(events).toNot(beNil())
                expect(events).to(haveCount(5))
                expect(events).to(containElementSatisfying({ (e: EventModel) in type(of: e) == CreateEventModel.self }))
                expect(events).to(containElementSatisfying({ (e: EventModel) in type(of: e) == PullRequestEventModel.self }))
                expect(events).to(containElementSatisfying({ (e: EventModel) in type(of: e) == WatchEventModel.self }))
                expect(events).to(containElementSatisfying({ (e: EventModel) in type(of: e) == ForkEventModel.self }))
            }
        }
    }
}

private let rawCreateRepoJson = "{ \u{22}id\u{22}: \u{22}5391454461\u{22}, \u{22}type\u{22}: \u{22}CreateEvent\u{22}, \u{22}actor\u{22}: { \u{22}id\u{22}: 4953175, \u{22}login\u{22}: \u{22}happylance\u{22}, \u{22}display_login\u{22}: \u{22}happylance\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/happylance\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/4953175?\u{22} }, \u{22}repo\u{22}: { \u{22}id\u{22}: 83169775, \u{22}name\u{22}: \u{22}happylance/ForecastIOClient\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/repos/happylance/ForecastIOClient\u{22} }, \u{22}payload\u{22}: { \u{22}ref\u{22}: null, \u{22}ref_type\u{22}: \u{22}repository\u{22}, \u{22}master_branch\u{22}: \u{22}master\u{22}, \u{22}description\u{22}: null, \u{22}pusher_type\u{22}: \u{22}user\u{22} }, \u{22}public\u{22}: true, \u{22}created_at\u{22}: \u{22}2017-02-25T23:22:34Z\u{22} }"
private let rawCreateTagJson = "{ \u{22}id\u{22}: \u{22}5391454459\u{22}, \u{22}type\u{22}: \u{22}CreateEvent\u{22}, \u{22}actor\u{22}: { \u{22}id\u{22}: 931435, \u{22}login\u{22}: \u{22}rtumaykin\u{22}, \u{22}display_login\u{22}: \u{22}rtumaykin\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/rtumaykin\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/931435?\u{22} }, \u{22}repo\u{22}: { \u{22}id\u{22}: 77345337, \u{22}name\u{22}: \u{22}rtumaykin/ssis-build\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/repos/rtumaykin/ssis-build\u{22} }, \u{22}payload\u{22}: { \u{22}ref\u{22}: \u{22}2.1.2\u{22}, \u{22}ref_type\u{22}: \u{22}tag\u{22}, \u{22}master_branch\u{22}: \u{22}master\u{22}, \u{22}description\u{22}: \u{22}A set of utilities that allow to autonomously build a Visual Studio SSIS project (dtproj) into a deployment package (ispac), and deploy the package to an SSIS catalog\u{22}, \u{22}pusher_type\u{22}: \u{22}user\u{22} }, \u{22}public\u{22}: true, \u{22}created_at\u{22}: \u{22}2017-02-25T23:22:34Z\u{22} }"
private let rawWatchJson = "{ \u{22}id\u{22}: \u{22}5391454441\u{22}, \u{22}type\u{22}: \u{22}WatchEvent\u{22}, \u{22}actor\u{22}: { \u{22}id\u{22}: 2153342, \u{22}login\u{22}: \u{22}chantron\u{22}, \u{22}display_login\u{22}: \u{22}chantron\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/chantron\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/2153342?\u{22} }, \u{22}repo\u{22}: { \u{22}id\u{22}: 30161665, \u{22}name\u{22}: \u{22}infernojs/inferno\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/repos/infernojs/inferno\u{22} }, \u{22}payload\u{22}: { \u{22}action\u{22}: \u{22}started\u{22} }, \u{22}public\u{22}: true, \u{22}created_at\u{22}: \u{22}2017-02-25T23:22:33Z\u{22}, \u{22}org\u{22}: { \u{22}id\u{22}: 14214240, \u{22}login\u{22}: \u{22}infernojs\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/orgs/infernojs\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/14214240?\u{22} } }"
private let rawPullRequestOpenedJson = "{ \u{22}id\u{22}: \u{22}5391495779\u{22}, \u{22}type\u{22}: \u{22}PullRequestEvent\u{22}, \u{22}actor\u{22}: { \u{22}id\u{22}: 20732208, \u{22}login\u{22}: \u{22}shahabhijeet\u{22}, \u{22}display_login\u{22}: \u{22}shahabhijeet\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/shahabhijeet\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/20732208?\u{22} }, \u{22}repo\u{22}: { \u{22}id\u{22}: 2928944, \u{22}name\u{22}: \u{22}Azure/azure-sdk-for-net\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net\u{22} }, \u{22}payload\u{22}: { \u{22}action\u{22}: \u{22}opened\u{22}, \u{22}number\u{22}: 2861, \u{22}pull_request\u{22}: { \u{22}url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861\u{22}, \u{22}id\u{22}: 107982535, \u{22}html_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net/pull/2861\u{22}, \u{22}diff_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net/pull/2861.diff\u{22}, \u{22}patch_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net/pull/2861.patch\u{22}, \u{22}issue_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/2861\u{22}, \u{22}number\u{22}: 2861, \u{22}state\u{22}: \u{22}open\u{22}, \u{22}locked\u{22}: false, \u{22}title\u{22}: \u{22}Updates for Network, NotificationHub, PowerBI\u{22}, \u{22}user\u{22}: { \u{22}login\u{22}: \u{22}shahabhijeet\u{22}, \u{22}id\u{22}: 20732208, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/20732208?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/shahabhijeet\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/shahabhijeet\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/received_events\u{22}, \u{22}type\u{22}: \u{22}User\u{22}, \u{22}site_admin\u{22}: false }, \u{22}body\u{22}: \u{22}netCore, VS2017 updates for Network, NotificationHub and PowerBI\u{22}, \u{22}created_at\u{22}: \u{22}2017-02-25T23:47:55Z\u{22}, \u{22}updated_at\u{22}: \u{22}2017-02-25T23:47:55Z\u{22}, \u{22}closed_at\u{22}: null, \u{22}merged_at\u{22}: null, \u{22}merge_commit_sha\u{22}: null, \u{22}assignee\u{22}: null, \u{22}assignees\u{22}: [  ], \u{22}milestone\u{22}: null, \u{22}commits_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861/commits\u{22}, \u{22}review_comments_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861/comments\u{22}, \u{22}review_comment_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/comments{/number}\u{22}, \u{22}comments_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/2861/comments\u{22}, \u{22}statuses_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/statuses/74ad44a396bd3d67104236f1848eb9c377bc76e0\u{22}, \u{22}head\u{22}: { \u{22}label\u{22}: \u{22}shahabhijeet:NPRps\u{22}, \u{22}ref\u{22}: \u{22}NPRps\u{22}, \u{22}sha\u{22}: \u{22}74ad44a396bd3d67104236f1848eb9c377bc76e0\u{22}, \u{22}user\u{22}: { \u{22}login\u{22}: \u{22}shahabhijeet\u{22}, \u{22}id\u{22}: 20732208, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/20732208?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/shahabhijeet\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/shahabhijeet\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/received_events\u{22}, \u{22}type\u{22}: \u{22}User\u{22}, \u{22}site_admin\u{22}: false }, \u{22}repo\u{22}: { \u{22}id\u{22}: 66677029, \u{22}name\u{22}: \u{22}azure-sdk-for-net\u{22}, \u{22}full_name\u{22}: \u{22}shahabhijeet/azure-sdk-for-net\u{22}, \u{22}owner\u{22}: { \u{22}login\u{22}: \u{22}shahabhijeet\u{22}, \u{22}id\u{22}: 20732208, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/20732208?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/shahabhijeet\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/shahabhijeet\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/shahabhijeet/received_events\u{22}, \u{22}type\u{22}: \u{22}User\u{22}, \u{22}site_admin\u{22}: false }, \u{22}private\u{22}: false, \u{22}html_url\u{22}: \u{22}https://github.com/shahabhijeet/azure-sdk-for-net\u{22}, \u{22}description\u{22}: \u{22}Microsoft Azure SDK for .NET\u{22}, \u{22}fork\u{22}: true, \u{22}url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net\u{22}, \u{22}forks_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/forks\u{22}, \u{22}keys_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/keys{/key_id}\u{22}, \u{22}collaborators_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/collaborators{/collaborator}\u{22}, \u{22}teams_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/teams\u{22}, \u{22}hooks_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/hooks\u{22}, \u{22}issue_events_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/issues/events{/number}\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/events\u{22}, \u{22}assignees_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/assignees{/user}\u{22}, \u{22}branches_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/branches{/branch}\u{22}, \u{22}tags_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/tags\u{22}, \u{22}blobs_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/git/blobs{/sha}\u{22}, \u{22}git_tags_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/git/tags{/sha}\u{22}, \u{22}git_refs_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/git/refs{/sha}\u{22}, \u{22}trees_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/git/trees{/sha}\u{22}, \u{22}statuses_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/statuses/{sha}\u{22}, \u{22}languages_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/languages\u{22}, \u{22}stargazers_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/stargazers\u{22}, \u{22}contributors_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/contributors\u{22}, \u{22}subscribers_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/subscribers\u{22}, \u{22}subscription_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/subscription\u{22}, \u{22}commits_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/commits{/sha}\u{22}, \u{22}git_commits_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/git/commits{/sha}\u{22}, \u{22}comments_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/comments{/number}\u{22}, \u{22}issue_comment_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/issues/comments{/number}\u{22}, \u{22}contents_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/contents/{+path}\u{22}, \u{22}compare_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/compare/{base}...{head}\u{22}, \u{22}merges_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/merges\u{22}, \u{22}archive_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/{archive_format}{/ref}\u{22}, \u{22}downloads_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/downloads\u{22}, \u{22}issues_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/issues{/number}\u{22}, \u{22}pulls_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/pulls{/number}\u{22}, \u{22}milestones_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/milestones{/number}\u{22}, \u{22}notifications_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/notifications{?since,all,participating}\u{22}, \u{22}labels_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/labels{/name}\u{22}, \u{22}releases_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/releases{/id}\u{22}, \u{22}deployments_url\u{22}: \u{22}https://api.github.com/repos/shahabhijeet/azure-sdk-for-net/deployments\u{22}, \u{22}created_at\u{22}: \u{22}2016-08-26T20:22:32Z\u{22}, \u{22}updated_at\u{22}: \u{22}2016-08-26T20:22:46Z\u{22}, \u{22}pushed_at\u{22}: \u{22}2017-02-25T23:45:11Z\u{22}, \u{22}git_url\u{22}: \u{22}git://github.com/shahabhijeet/azure-sdk-for-net.git\u{22}, \u{22}ssh_url\u{22}: \u{22}git@github.com:shahabhijeet/azure-sdk-for-net.git\u{22}, \u{22}clone_url\u{22}: \u{22}https://github.com/shahabhijeet/azure-sdk-for-net.git\u{22}, \u{22}svn_url\u{22}: \u{22}https://github.com/shahabhijeet/azure-sdk-for-net\u{22}, \u{22}homepage\u{22}: \u{22}http://www.windowsazure.com/en-us/develop/net/\u{22}, \u{22}size\u{22}: 121318, \u{22}stargazers_count\u{22}: 0, \u{22}watchers_count\u{22}: 0, \u{22}language\u{22}: \u{22}C#\u{22}, \u{22}has_issues\u{22}: false, \u{22}has_downloads\u{22}: true, \u{22}has_wiki\u{22}: true, \u{22}has_pages\u{22}: false, \u{22}forks_count\u{22}: 0, \u{22}mirror_url\u{22}: null, \u{22}open_issues_count\u{22}: 0, \u{22}forks\u{22}: 0, \u{22}open_issues\u{22}: 0, \u{22}watchers\u{22}: 0, \u{22}default_branch\u{22}: \u{22}AutoRest\u{22} } }, \u{22}base\u{22}: { \u{22}label\u{22}: \u{22}Azure:AutoRest\u{22}, \u{22}ref\u{22}: \u{22}AutoRest\u{22}, \u{22}sha\u{22}: \u{22}4d37cc15467d2e44f84ac5e64848df88d0d72142\u{22}, \u{22}user\u{22}: { \u{22}login\u{22}: \u{22}Azure\u{22}, \u{22}id\u{22}: 6844498, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/6844498?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/Azure\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/Azure\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/Azure/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/Azure/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/Azure/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/Azure/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/Azure/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/Azure/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/Azure/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/Azure/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/Azure/received_events\u{22}, \u{22}type\u{22}: \u{22}Organization\u{22}, \u{22}site_admin\u{22}: false }, \u{22}repo\u{22}: { \u{22}id\u{22}: 2928944, \u{22}name\u{22}: \u{22}azure-sdk-for-net\u{22}, \u{22}full_name\u{22}: \u{22}Azure/azure-sdk-for-net\u{22}, \u{22}owner\u{22}: { \u{22}login\u{22}: \u{22}Azure\u{22}, \u{22}id\u{22}: 6844498, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/6844498?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/Azure\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/Azure\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/Azure/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/Azure/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/Azure/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/Azure/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/Azure/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/Azure/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/Azure/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/Azure/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/Azure/received_events\u{22}, \u{22}type\u{22}: \u{22}Organization\u{22}, \u{22}site_admin\u{22}: false }, \u{22}private\u{22}: false, \u{22}html_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net\u{22}, \u{22}description\u{22}: \u{22}Azure Tools for Visual Studio\u{22}, \u{22}fork\u{22}: false, \u{22}url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net\u{22}, \u{22}forks_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/forks\u{22}, \u{22}keys_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/keys{/key_id}\u{22}, \u{22}collaborators_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/collaborators{/collaborator}\u{22}, \u{22}teams_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/teams\u{22}, \u{22}hooks_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/hooks\u{22}, \u{22}issue_events_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/events{/number}\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/events\u{22}, \u{22}assignees_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/assignees{/user}\u{22}, \u{22}branches_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/branches{/branch}\u{22}, \u{22}tags_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/tags\u{22}, \u{22}blobs_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/git/blobs{/sha}\u{22}, \u{22}git_tags_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/git/tags{/sha}\u{22}, \u{22}git_refs_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/git/refs{/sha}\u{22}, \u{22}trees_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/git/trees{/sha}\u{22}, \u{22}statuses_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/statuses/{sha}\u{22}, \u{22}languages_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/languages\u{22}, \u{22}stargazers_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/stargazers\u{22}, \u{22}contributors_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/contributors\u{22}, \u{22}subscribers_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/subscribers\u{22}, \u{22}subscription_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/subscription\u{22}, \u{22}commits_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/commits{/sha}\u{22}, \u{22}git_commits_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/git/commits{/sha}\u{22}, \u{22}comments_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/comments{/number}\u{22}, \u{22}issue_comment_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/comments{/number}\u{22}, \u{22}contents_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/contents/{+path}\u{22}, \u{22}compare_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/compare/{base}...{head}\u{22}, \u{22}merges_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/merges\u{22}, \u{22}archive_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/{archive_format}{/ref}\u{22}, \u{22}downloads_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/downloads\u{22}, \u{22}issues_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues{/number}\u{22}, \u{22}pulls_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls{/number}\u{22}, \u{22}milestones_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/milestones{/number}\u{22}, \u{22}notifications_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/notifications{?since,all,participating}\u{22}, \u{22}labels_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/labels{/name}\u{22}, \u{22}releases_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/releases{/id}\u{22}, \u{22}deployments_url\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/deployments\u{22}, \u{22}created_at\u{22}: \u{22}2011-12-06T23:32:47Z\u{22}, \u{22}updated_at\u{22}: \u{22}2017-02-25T15:52:52Z\u{22}, \u{22}pushed_at\u{22}: \u{22}2017-02-25T21:00:58Z\u{22}, \u{22}git_url\u{22}: \u{22}git://github.com/Azure/azure-sdk-for-net.git\u{22}, \u{22}ssh_url\u{22}: \u{22}git@github.com:Azure/azure-sdk-for-net.git\u{22}, \u{22}clone_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net.git\u{22}, \u{22}svn_url\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net\u{22}, \u{22}homepage\u{22}: \u{22}https://www.visualstudio.com/features/azure-tools-vs\u{22}, \u{22}size\u{22}: 134130, \u{22}stargazers_count\u{22}: 885, \u{22}watchers_count\u{22}: 885, \u{22}language\u{22}: \u{22}C#\u{22}, \u{22}has_issues\u{22}: true, \u{22}has_downloads\u{22}: true, \u{22}has_wiki\u{22}: true, \u{22}has_pages\u{22}: false, \u{22}forks_count\u{22}: 742, \u{22}mirror_url\u{22}: null, \u{22}open_issues_count\u{22}: 192, \u{22}forks\u{22}: 742, \u{22}open_issues\u{22}: 192, \u{22}watchers\u{22}: 885, \u{22}default_branch\u{22}: \u{22}AutoRest\u{22} } }, \u{22}_links\u{22}: { \u{22}self\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861\u{22} }, \u{22}html\u{22}: { \u{22}href\u{22}: \u{22}https://github.com/Azure/azure-sdk-for-net/pull/2861\u{22} }, \u{22}issue\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/2861\u{22} }, \u{22}comments\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/issues/2861/comments\u{22} }, \u{22}review_comments\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861/comments\u{22} }, \u{22}review_comment\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/comments{/number}\u{22} }, \u{22}commits\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/pulls/2861/commits\u{22} }, \u{22}statuses\u{22}: { \u{22}href\u{22}: \u{22}https://api.github.com/repos/Azure/azure-sdk-for-net/statuses/74ad44a396bd3d67104236f1848eb9c377bc76e0\u{22} } }, \u{22}merged\u{22}: false, \u{22}mergeable\u{22}: null, \u{22}mergeable_state\u{22}: \u{22}unknown\u{22}, \u{22}merged_by\u{22}: null, \u{22}comments\u{22}: 0, \u{22}review_comments\u{22}: 0, \u{22}maintainer_can_modify\u{22}: true, \u{22}commits\u{22}: 1, \u{22}additions\u{22}: 61, \u{22}deletions\u{22}: 90, \u{22}changed_files\u{22}: 7 } }, \u{22}public\u{22}: true, \u{22}created_at\u{22}: \u{22}2017-02-25T23:47:56Z\u{22}, \u{22}org\u{22}: { \u{22}id\u{22}: 6844498, \u{22}login\u{22}: \u{22}Azure\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/orgs/Azure\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/6844498?\u{22} } }"
private let rawForkJson = "{ \u{22}id\u{22}: \u{22}5391468192\u{22}, \u{22}type\u{22}: \u{22}ForkEvent\u{22}, \u{22}actor\u{22}: { \u{22}id\u{22}: 1623538, \u{22}login\u{22}: \u{22}CNXTEoE\u{22}, \u{22}display_login\u{22}: \u{22}CNXTEoE\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/CNXTEoE\u{22}, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/1623538?\u{22} }, \u{22}repo\u{22}: { \u{22}id\u{22}: 18710823, \u{22}name\u{22}: \u{22}mekkablue/MakeCorner\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/repos/mekkablue/MakeCorner\u{22} }, \u{22}payload\u{22}: { \u{22}forkee\u{22}: { \u{22}id\u{22}: 83170201, \u{22}name\u{22}: \u{22}MakeCorner\u{22}, \u{22}full_name\u{22}: \u{22}CNXTEoE/MakeCorner\u{22}, \u{22}owner\u{22}: { \u{22}login\u{22}: \u{22}CNXTEoE\u{22}, \u{22}id\u{22}: 1623538, \u{22}avatar_url\u{22}: \u{22}https://avatars.githubusercontent.com/u/1623538?v=3\u{22}, \u{22}gravatar_id\u{22}: \u{22}\u{22}, \u{22}url\u{22}: \u{22}https://api.github.com/users/CNXTEoE\u{22}, \u{22}html_url\u{22}: \u{22}https://github.com/CNXTEoE\u{22}, \u{22}followers_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/followers\u{22}, \u{22}following_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/following{/other_user}\u{22}, \u{22}gists_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/gists{/gist_id}\u{22}, \u{22}starred_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/starred{/owner}{/repo}\u{22}, \u{22}subscriptions_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/subscriptions\u{22}, \u{22}organizations_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/orgs\u{22}, \u{22}repos_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/repos\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/events{/privacy}\u{22}, \u{22}received_events_url\u{22}: \u{22}https://api.github.com/users/CNXTEoE/received_events\u{22}, \u{22}type\u{22}: \u{22}User\u{22}, \u{22}site_admin\u{22}: false }, \u{22}private\u{22}: false, \u{22}html_url\u{22}: \u{22}https://github.com/CNXTEoE/MakeCorner\u{22}, \u{22}description\u{22}: \u{22}Glyphs Filter for turning curved segments into corners.\u{22}, \u{22}fork\u{22}: true, \u{22}url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner\u{22}, \u{22}forks_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/forks\u{22}, \u{22}keys_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/keys{/key_id}\u{22}, \u{22}collaborators_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/collaborators{/collaborator}\u{22}, \u{22}teams_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/teams\u{22}, \u{22}hooks_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/hooks\u{22}, \u{22}issue_events_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/issues/events{/number}\u{22}, \u{22}events_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/events\u{22}, \u{22}assignees_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/assignees{/user}\u{22}, \u{22}branches_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/branches{/branch}\u{22}, \u{22}tags_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/tags\u{22}, \u{22}blobs_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/git/blobs{/sha}\u{22}, \u{22}git_tags_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/git/tags{/sha}\u{22}, \u{22}git_refs_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/git/refs{/sha}\u{22}, \u{22}trees_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/git/trees{/sha}\u{22}, \u{22}statuses_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/statuses/{sha}\u{22}, \u{22}languages_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/languages\u{22}, \u{22}stargazers_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/stargazers\u{22}, \u{22}contributors_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/contributors\u{22}, \u{22}subscribers_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/subscribers\u{22}, \u{22}subscription_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/subscription\u{22}, \u{22}commits_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/commits{/sha}\u{22}, \u{22}git_commits_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/git/commits{/sha}\u{22}, \u{22}comments_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/comments{/number}\u{22}, \u{22}issue_comment_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/issues/comments{/number}\u{22}, \u{22}contents_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/contents/{+path}\u{22}, \u{22}compare_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/compare/{base}...{head}\u{22}, \u{22}merges_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/merges\u{22}, \u{22}archive_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/{archive_format}{/ref}\u{22}, \u{22}downloads_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/downloads\u{22}, \u{22}issues_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/issues{/number}\u{22}, \u{22}pulls_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/pulls{/number}\u{22}, \u{22}milestones_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/milestones{/number}\u{22}, \u{22}notifications_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/notifications{?since,all,participating}\u{22}, \u{22}labels_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/labels{/name}\u{22}, \u{22}releases_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/releases{/id}\u{22}, \u{22}deployments_url\u{22}: \u{22}https://api.github.com/repos/CNXTEoE/MakeCorner/deployments\u{22}, \u{22}created_at\u{22}: \u{22}2017-02-25T23:30:31Z\u{22}, \u{22}updated_at\u{22}: \u{22}2017-01-05T16:25:03Z\u{22}, \u{22}pushed_at\u{22}: \u{22}2016-12-08T14:02:59Z\u{22}, \u{22}git_url\u{22}: \u{22}git://github.com/CNXTEoE/MakeCorner.git\u{22}, \u{22}ssh_url\u{22}: \u{22}git@github.com:CNXTEoE/MakeCorner.git\u{22}, \u{22}clone_url\u{22}: \u{22}https://github.com/CNXTEoE/MakeCorner.git\u{22}, \u{22}svn_url\u{22}: \u{22}https://github.com/CNXTEoE/MakeCorner\u{22}, \u{22}homepage\u{22}: null, \u{22}size\u{22}: 41, \u{22}stargazers_count\u{22}: 0, \u{22}watchers_count\u{22}: 0, \u{22}language\u{22}: null, \u{22}has_issues\u{22}: false, \u{22}has_downloads\u{22}: true, \u{22}has_wiki\u{22}: true, \u{22}has_pages\u{22}: false, \u{22}forks_count\u{22}: 0, \u{22}mirror_url\u{22}: null, \u{22}open_issues_count\u{22}: 0, \u{22}forks\u{22}: 0, \u{22}open_issues\u{22}: 0, \u{22}watchers\u{22}: 0, \u{22}default_branch\u{22}: \u{22}master\u{22}, \u{22}public\u{22}: true } }, \u{22}public\u{22}: true, \u{22}created_at\u{22}: \u{22}2017-02-25T23:30:31Z\u{22} }"
